@@ -23,6 +23,7 @@
 
 #include "ephy-embed-shell.h"
 #include "ephy-embed-prefs.h"
+#include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
 #include "ephy-prefs.h"
@@ -454,7 +455,7 @@ history_service_query_urls_cb (EphyHistoryService     *history,
                           "<body>\n",
                           lang, lang,
                           ((gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) ? "rtl" : "ltr"),
-                          _("Most Visited"));
+                          _(OVERVIEW_PAGE_TITLE));
   g_free (lang);
 
   if (g_list_length (urls) == 0 || !success) {
@@ -575,7 +576,7 @@ ephy_about_handler_handle_incognito (EphyAboutHandler       *handler,
                           gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL ? "rtl" : "ltr",
                           _("Private Browsing"),
                           _("Private Browsing"),
-                          _("You are currently browsing <em>incognito</em>. Pages viewed in this "
+                          _("You are currently browsing incognito. Pages viewed in this "
                             "mode will not show up in your browsing history and all stored "
                             "information will be cleared when you close the window. Files you "
                             "download will be kept."),
