@@ -107,7 +107,7 @@ combo_box_changed_cb (GtkComboBox *combo_box,
 
   active = gtk_combo_box_get_active (combo_box);
   if (active == 0)
-    gtk_button_set_label (button, _("Ch_oose File"));
+    gtk_button_set_label (button, _("Ch_oose File…"));
   else if (active == 1)
     gtk_button_set_label (button, _("I_mport"));
 }
@@ -159,22 +159,18 @@ get_firefox_profiles (void)
   channel = g_io_channel_new_file (filename, "r", NULL);
   g_free (filename);
 
-  if (channel) {
-    do {
-      g_io_channel_read_line (channel, &line, &length, NULL, NULL);
+  do {
+    g_io_channel_read_line (channel, &line, &length, NULL, NULL);
 
-      profile = g_strdup_printf ("[Profile%d]\n", count);
-      if (g_strcmp0 (line, profile) == 0) {
-        profiles = g_slist_append (profiles, get_path (channel));
+    profile = g_strdup_printf ("[Profile%d]\n", count);
+    if (g_strcmp0 (line, profile) == 0) {
+      profiles = g_slist_append (profiles, get_path (channel));
 
-        count++;
-      }
-      g_free (profile);
-      g_free (line);
-    } while (length != 0);
-
-    g_io_channel_unref (channel);
-  }
+      count++;
+    }
+    g_free (profile);
+    g_free (line);
+  } while (length != 0);
 
   return profiles;
 }
@@ -848,6 +844,23 @@ void window_cmd_combined_stop_reload (GSimpleAction *action,
   g_variant_unref (state);
 }
 
+void
+window_cmd_page_menu (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       user_data)
+
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyHeaderBar *header_bar;
+  GtkMenuButton *button;
+  GtkPopover *popover;
+
+  header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (window));
+  button = GTK_MENU_BUTTON (ephy_header_bar_get_page_menu_button (header_bar));
+  popover = gtk_menu_button_get_popover (button);
+  gtk_popover_popup (popover);
+}
+
 void
 window_cmd_new_tab (GSimpleAction *action,
                     GVariant      *parameter,
